# tigrbl_billing.views – ORM-like, table-bound OLAP views (read-only)
from __future__ import annotations

from .mrr_subscription import VwMRRSubscription
from .arr_customer import VwARRCustomer
from .revenue_by_split_rule import VwRevenueBySplitRule
from .cohort_retention import VwCohortRetention
from .arpu_monthly import VwARPUMonthly
from .dunning_funnel import VwDunningFunnel
from .usage_coverage_ratio import VwUsageCoverageRatio

__all__ = [
    "VwMRRSubscription",
    "VwARRCustomer",
    "VwRevenueBySplitRule",
    "VwCohortRetention",
    "VwARPUMonthly",
    "VwDunningFunnel",
    "VwUsageCoverageRatio",
]


def all_views():
    return (
        VwMRRSubscription,
        VwARRCustomer,
        VwRevenueBySplitRule,
        VwCohortRetention,
        VwARPUMonthly,
        VwDunningFunnel,
        VwUsageCoverageRatio,
    )


def __dir__():
    return sorted(__all__)
